@@ -24,14 +24,10 @@ use core::{
 use utils::{
     collections::Vec,
     string::{String, ToString},
-    AsBytes, ByteReader, ByteWriter, Deserializable, DeserializationError, Serializable,
+    AsBytes, ByteReader, ByteWriter, Deserializable, DeserializationError, Randomizable,
+    Serializable,
 };
 
-#[cfg(feature = "std")]
-use core::ops::Range;
-#[cfg(feature = "std")]
-use rand::{distributions::Uniform, prelude::*};
-
 #[cfg(test)]
 mod tests;
 
@@ -44,9 +40,6 @@ const M: u128 = 340282366920938463463374557953744961537;
 // 2^40 root of unity
 const G: u128 = 23953097886125630542083529559205016746;
 
-#[cfg(feature = "std")]
-const RANGE: Range<u128> = Range { start: 0, end: M };
-
 // Number of bytes needed to represent field element
 const ELEMENT_BYTES: usize = core::mem::size_of::<u128>();
 
@@ -78,7 +71,7 @@ impl FieldElement for BaseElement {
 
     const ELEMENT_BYTES: usize = ELEMENT_BYTES;
 
-    const IS_MALLEABLE: bool = false;
+    const IS_CANONICAL: bool = true;
 
     fn inv(self) -> Self {
         BaseElement(inv(self.0))
@@ -88,25 +81,6 @@ impl FieldElement for BaseElement {
         BaseElement(self.0)
     }
 
-    #[cfg(feature = "std")]
-    fn rand() -> Self {
-        let range = Uniform::from(RANGE);
-        let mut g = thread_rng();
-        BaseElement(g.sample(range))
-    }
-
-    fn from_random_bytes(bytes: &[u8]) -> Option<Self> {
-        Self::try_from(bytes).ok()
-    }
-
-    fn elements_into_bytes(elements: Vec<Self>) -> Vec<u8> {
-        let mut v = core::mem::ManuallyDrop::new(elements);
-        let p = v.as_mut_ptr();
-        let len = v.len() * Self::ELEMENT_BYTES;
-        let cap = v.capacity() * Self::ELEMENT_BYTES;
-        unsafe { Vec::from_raw_parts(p as *mut u8, len, cap) }
-    }
-
     fn elements_as_bytes(elements: &[Self]) -> &[u8] {
         // TODO: take endianness into account
         let p = elements.as_ptr();
@@ -150,15 +124,8 @@ impl FieldElement for BaseElement {
         unsafe { Vec::from_raw_parts(p as *mut Self, len, cap) }
     }
 
-    #[cfg(feature = "std")]
-    fn prng_vector(seed: [u8; 32], n: usize) -> Vec<Self> {
-        let range = Uniform::from(RANGE);
-        let g = StdRng::from_seed(seed);
-        g.sample_iter(range).take(n).map(BaseElement).collect()
-    }
-
-    fn normalize(&mut self) {
-        // do nothing since the internal and canonical representations are the same
+    fn as_base_elements(elements: &[Self]) -> &[Self::BaseField] {
+        elements
     }
 }
 
@@ -195,6 +162,14 @@ impl StarkField for BaseElement {
     }
 }
 
+impl Randomizable for BaseElement {
+    const VALUE_SIZE: usize = Self::ELEMENT_BYTES;
+
+    fn from_random_bytes(bytes: &[u8]) -> Option<Self> {
+        Self::try_from(bytes).ok()
+    }
+}
+
 impl Display for BaseElement {
     fn fmt(&self, f: &mut Formatter) -> core::fmt::Result {
         write!(f, "{}", self.0)
